@@ -1,5 +1,5 @@
 # This file is part of tdmclient.
-# Copyright 2021 ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE,
+# Copyright 2021-2022 ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE,
 # Miniature Mobile Robots group, Switzerland
 # Author: Yves Piguet
 #
@@ -12,15 +12,20 @@ Communication with TDM.
 from tdmclient.fb import FlatBuffer, Table, Union
 from tdmclient.zeroconf import TDMZeroconfBrowser
 from tdmclient.tcp import TDMConnection
+try:
+    from tdmclient.ws import TDMConnectionWS
+except ModuleNotFoundError:
+    pass
 from tdmclient.thymio import ThymioFB, Node
 from tdmclient.client import Client
-from tdmclient.clientasync import ClientAsync
+from tdmclient.clientasync import ClientAsync, NodeLockError
 from tdmclient.clientnode import ClientNode
 from tdmclient.clientasyncnode import ClientAsyncNode
 from tdmclient.clientasynccachenode import ClientAsyncCacheNode, ArrayCache
 from tdmclient.repl import TDMConsole
 
-from tdmclient.server import Server, ServerNode
+from tdmclient.server import (Server, ServerNode,
+                              ServerRawTDMHandler, ServerHandler)
 
 # shortcut
 aw = ClientAsync.aw
