@@ -21,13 +21,19 @@ class TranspilerError(Exception):
         self.message = message
         self.ast_node = ast_node
         self.syntax_error = syntax_error
+        self.lineno = (self.ast_node.lineno if ast_node is not None
+                       else self.syntax_error.args[1][1] if self.syntax_error
+                       else None)
 
     def __str__(self):
-        output = self.message
-        if self.ast_node is not None:
-            output += f" (line {self.ast_node.lineno})"
-        elif self.syntax_error is not None:
-            output += f" (line {self.syntax_error.args[1][1]})"
+        output = ""
+        if self.syntax_error is not None:
+            # show context like in Python repl
+            output += "\n  " + self.syntax_error.text
+            output += (1 + self.syntax_error.offset) * " " + "^\n"
+        output += self.message
+        if self.lineno is not None:
+            output += f" (line {self.lineno})"
         return output
 
 
@@ -101,7 +107,7 @@ class Context:
         if not self.collect_local_variables and is_target:
             if "." in name:
                 module_name, symbol = name.split(".", 1)
-                module_and_name = context.get_module(module_name), symbol
+                module_and_name = context.get_module(module_name, ast_node=ast_node), symbol
             else:
                 module_and_name = context.get_module_for_symbol(name)
             if module_and_name is not None:
@@ -110,7 +116,7 @@ class Context:
                     raise TranspilerError(f"assignment to constant {name}",
                                           ast_node=ast_node)
 
-        module_value = context.get_module_value(name, not is_target)
+        module_value = context.get_module_value(name, not is_target, ast_node=ast_node)
         if module_value:
             return module_value
         if self.function_name is None or is_global:
@@ -153,12 +159,12 @@ class Context:
             else:
                 context.var[name] = size
 
-    def get_module_value(self, name, ancestors=False):
+    def get_module_value(self, name, ancestors=False, ast_node=None):
         """Get the value of a variable or constant defined in an imported module.
         """
         if "." in name:
             module_name, name = name.split(".", 1)
-            module = self.get_module(module_name)
+            module = self.get_module(module_name, ast_node=ast_node)
         else:
             module_and_name = self.get_module_for_symbol(name, ancestors)
             if module_and_name is None:
@@ -166,12 +172,12 @@ class Context:
             module, name = module_and_name
         return module.constants[name][0] if name in module.constants else module.variables[name][0] if name in module.variables else None
 
-    def get_module_function(self, name):
+    def get_module_function(self, name, ast_node=None):
         """Get a function defined in an imported module.
         """
         if "." in name:
             module_name, name = name.split(".", 1)
-            module = self.get_module(module_name)
+            module = self.get_module(module_name, ast_node=ast_node)
         else:
             module_and_name = self.get_module_for_symbol(name, True)
             if module_and_name is None:
@@ -179,15 +185,16 @@ class Context:
             module, name = module_and_name
         return module.functions[name] if name in module.functions else None
 
-    def get_module(self, module_name):
+    def get_module(self, module_name, ast_node=None):
         """Return module in current context or ancestor.
         """
         if module_name in self.modules:
             return self.modules[module_name]
         elif self.parent_context is not None:
-            return self.parent_context.get_module(module_name)
+            return self.parent_context.get_module(module_name, ast_node)
         else:
-            raise NameError(f"name '{module_name}' is not defined")
+            raise TranspilerError(f"name '{module_name}' is not defined",
+                                  ast_node=ast_node)
 
     def get_module_for_symbol(self, symbol, ancestors=False):
         """Return module and name where symbol is defined in current context, and
@@ -197,13 +204,13 @@ class Context:
                 else self.parent_context.get_module_for_symbol(symbol, True) if ancestors and self.parent_context is not None
                 else None)
 
-    def var_array_size(self, name, is_target=False):
+    def var_array_size(self, name, is_target=False, ast_node=None):
         """Return size if name is a local or global array variable,
         None if it is a local or global scalar, or False if unknown.
         """
         if "." in name:
             module_name, name = name.split(".", 1)
-            module = self.get_module(module_name)
+            module = self.get_module(module_name, ast_node=ast_node)
             return (module.constants[name][1] if name in module.constants
                     else module.variables[name][1] if name in module.variables
                     else False)
@@ -314,7 +321,7 @@ class Context:
             if len(node.args) != 1:
                 return None
             if isinstance(node.args[0], ast.Name):
-                return self.var_array_size(node.args[0].id)
+                return self.var_array_size(node.args[0].id, ast_node=node)
             else:
                 return self.list_length(node.args[0])
 
@@ -385,7 +392,8 @@ class AFunction:
             if self.argin[i]:
                 # array arg
                 if isinstance(arg, ast.Name):
-                    arg_code.append(arg.id)
+                    value = context.var_str(arg.id, False, arg)
+                    arg_code.append(value)
                 elif context.list_length(arg) is not None:
                     value, aux_st, _ = atranspiler.compile_expr(arg, context, atranspiler.PRI_ASSIGN)
                     aux_statements += aux_st
@@ -439,6 +447,7 @@ class ATranspiler:
         self.src = None
         self.ast_preamble = None
         self.ast = None
+        self.context_top = None
         self.output_src = None
 
         # fun_name: AFunction (empty, moved to thymio module)
@@ -475,6 +484,7 @@ class ATranspiler:
         """
         self.src = source
         self.ast = None
+        self.context_top = None
         self.output_src = None
         self.print_format_strings = {}
         self.print_format_string_next_id = 0
@@ -667,8 +677,11 @@ end
                     # no return value: must not be called in a subexpression
                     if priority_container != self.PRI_EXPR:
                         raise TranspilerError("function without return value called in an expression", ast_node=node)
+                else:
+                    # has_return_val unknown yet (first compiler phase)
+                    code = "?"  # dummy generated code
                 return code, aux_statements, False
-            a_function = context.get_module_function(fun_name)
+            a_function = context.get_module_function(fun_name, ast_node=node)
             if a_function is None and fun_name in self.predefined_function_dict:
                 a_function = self.predefined_function_dict[fun_name]
             if a_function is not None:
@@ -694,7 +707,7 @@ end
                 if len(node.args) != 1:
                     raise TranspilerError("wrong number of arguments for len", ast_node=node)
                 if isinstance(node.args[0], ast.Name):
-                    len_arg_size = context.var_array_size(node.args[0].id)
+                    len_arg_size = context.var_array_size(node.args[0].id, ast_node=node)
                     code = f"{len_arg_size}" if len_arg_size is not None else "0"
                 else:
                     list_len = context.list_length(node.args[0])
@@ -784,16 +797,17 @@ end
             code, aux_st, is_boolean = self.compile_expr(node.value, context, self.PRI_NUMERIC)
             aux_statements += aux_st
             return code, aux_statements, False
-        elif isinstance(node, ast.Name):
-            var_array_size = context.var_array_size(node.id)
+        elif isinstance(node, (ast.Name, ast.Attribute)):
+            name = self.decode_attr(node)
+            var_array_size = context.var_array_size(name, ast_node=node)
             if var_array_size is False:
-                raise TranspilerError(f"unknown variable '{node.id}'", node)
+                raise TranspilerError(f"unknown variable '{name}'", node)
             if isinstance(var_array_size, int):
-                raise TranspilerError(f"list variable '{node.id}' used in expression", node)
-            code = context.var_str(node.id, ast_node=node)
+                raise TranspilerError(f"list variable '{name}' used in expression", node)
+            code = context.var_str(name, ast_node=node)
         elif isinstance(node, ast.Subscript):
             name = self.decode_attr(node.value)
-            var_array_size = context.var_array_size(name)
+            var_array_size = context.var_array_size(name, ast_node=node)
             if var_array_size is False:
                 raise TranspilerError(f"unknown variable '{name}'", node)
             if var_array_size is None:
@@ -887,12 +901,12 @@ end
                 elif isinstance(node.value, (ast.Name, ast.Attribute)):
                     # var1 = var2: inherit size
                     name_right = self.decode_attr(node.value)
-                    target_size = context.var_array_size(name_right, is_target=False)
+                    target_size = context.var_array_size(name_right, is_target=False, ast_node=node)
                     if target_size is False:
                         raise TranspilerError(f"unknown variable '{name_right}'", node)
                     context.declare_var(target, target_size, ast_node=node)
                     # code special case (parsing of var2 as an expression would fail)
-                    module_value = context.get_module_value(name_right, True)
+                    module_value = context.get_module_value(name_right, True, ast_node=node)
                     if module_value:
                         return f"""{target_str} = {module_value}
 """
@@ -936,7 +950,7 @@ end
 """
             return code
         if isinstance(node, ast.AugAssign):
-            op_str = {
+            op_dict = {
                 ast.Add: "+",
                 ast.BitAnd: "&",
                 ast.BitOr: "|",
@@ -947,8 +961,24 @@ end
                 ast.Mult: "*",
                 ast.RShift: ">>",
                 ast.Sub: "-",
-            }[type(node.op)]
+            }
+            if type(node.op) not in op_dict:
+                print(type(node.op))
+                op_str = {
+                    ast.MatMult: "@",
+                    ast.Div: "/",
+                    ast.Pow: "**"
+                }[type(node.op)]
+                raise TranspilerError(f"unsupported augmented assignment '{op_str}='", node)
+            op_str = op_dict[type(node.op)]
             target, index = self.decode_target(node.target)
+            var_array_size = context.var_array_size(target, ast_node=node)
+            if var_array_size is False:
+                raise TranspilerError(f"unknown variable '{target}' in augmented assignment", node)
+            elif var_array_size is None and index is not None:
+                raise TranspilerError(f"indexing of scalar variable '{target}' in augmented assignment", node)
+            elif var_array_size is not None and index is None:
+                raise TranspilerError(f"list variable '{target}' in augmented assignment", node)
             target_str = context.var_str(target, ast_node=node)
             value, aux_statements, is_boolean = self.compile_expr(node.value, context, self.PRI_NUMERIC)
             code += aux_statements
@@ -1253,7 +1283,7 @@ return
         """
 
         # top-level context
-        context_top = Context()
+        self.context_top = Context()
 
         # parse and split into top code and function definitions
         try:
@@ -1261,11 +1291,11 @@ return
             self.ast = ast.parse(self.src)
         except SyntaxError as error:
             raise TranspilerError(error.args[0], syntax_error=error) from None
-        top_code = self.split(context_top)
+        top_code = self.split(self.context_top)
 
         # compile top-level code (first pass for global variable sizes)
         self.reset_transpile_phase()
-        self.output_src = self.compile_node_array(top_code, context_top)
+        self.output_src = self.compile_node_array(top_code, self.context_top)
 
         # reset for next phase
         self.reset_transpile_phase()
@@ -1274,17 +1304,17 @@ return
         function_src = ""
         # first pass to gather local variables into context_fun[.] (not declared global, but assigned to)
         # function return type isn't known yet (context.has_ret_value = None doesn't raise exceptions)
-        for fun_name in context_top.functions:
+        for fun_name in self.context_top.functions:
             fun_print_format_string_next_id = self.print_format_string_next_id
-            context_top.functions[fun_name].collect_local_variables = True
-            _ = self.compile_node_array(context_top.functions[fun_name].function_def.body, context_top.functions[fun_name])
-            context_top.functions[fun_name].collect_local_variables = False
+            self.context_top.functions[fun_name].collect_local_variables = True
+            _ = self.compile_node_array(self.context_top.functions[fun_name].function_def.body, self.context_top.functions[fun_name])
+            self.context_top.functions[fun_name].collect_local_variables = False
             # set functions without return statements to void
-            context_top.functions[fun_name].freeze_return_type()
+            self.context_top.functions[fun_name].freeze_return_type()
             self.reset_transpile_phase(fun_print_format_string_next_id)
         # second pass to produce transpiled code with correct local variable names
-        for fun_name in context_top.functions:
-            function = context_top.functions[fun_name]
+        for fun_name in self.context_top.functions:
+            function = self.context_top.functions[fun_name]
             fun_output_src = self.compile_node_array(function.function_def.body, function)
             if function.is_onevent:
                 self.events_in[fun_name] = len(function.function_def.args.args)
@@ -1303,25 +1333,25 @@ sub {fun_name}
             function_src += fun_output_src
         # onevent_preamble for functions not defined in Python source code
         for fun_name in self.onevent_preamble:
-            if fun_name not in context_top.functions:
+            if fun_name not in self.context_top.functions:
                 function_src += f"""
 onevent {fun_name.replace("_", ".")}
 """
                 function_src += "".join(self.onevent_preamble[fun_name])
 
         # compile top-level code again, now that function return types are known
-        self.output_src = self.compile_node_array(top_code, context_top) + function_src
+        self.output_src = self.compile_node_array(top_code, self.context_top) + function_src
 
         # check recursivity
-        for fun_name in context_top.functions:
-            if context_top.functions[fun_name].is_recursive(context_top.functions):
-                raise TranspilerError(f"recursive function '{fun_name}'", context_top.functions[fun_name].function_def)
+        for fun_name in self.context_top.functions:
+            if self.context_top.functions[fun_name].is_recursive(self.context_top.functions):
+                raise TranspilerError(f"recursive function '{fun_name}'", self.context_top.functions[fun_name].function_def)
 
         # variable declarations
-        var_decl = context_top.var_declarations()
+        var_decl = self.context_top.var_declarations()
         var_decl += "".join([
-            context_top.functions[fun_name].var_declarations()
-            for fun_name in context_top.functions
+            self.context_top.functions[fun_name].var_declarations()
+            for fun_name in self.context_top.functions
         ])
         for var_name in self.additional_var_declarations:
             var_decl += f"""var {var_name} = {self.additional_var_declarations[var_name]}
