@@ -87,22 +87,25 @@ async def list(zeroconf=None, tdm_addr=None, tdm_port=None, password=None,
                 print(f"firmware: {node.props['fw_version']}")
             print()
 
-async def start(zeroconf=None, tdm_addr=None, tdm_port=None, password=None,
+async def start(zeroconf=False, zeroconf_all=False,
+                tdm_ws=False, tdm_addr=None, tdm_port=None,
+                password=None,
                 debug=0, **kwargs):
     """Start the connection with the Thymio and variable synchronization.
 
     Arguments:
-        tdm_addr - TDM address as a string (default: localhost)
-        tdm_port - TDM TCP port number
-                   (default: standard or provided by zeroconf)
-        password - TDM password (default: None, not necessary for local TDM)
+        tdm_addr - TDM address (default: provided by zeroconf or local)
+        tdm_port - TDM port (default: provided by zeroconf, or 8596 (tcp) or 8597 (ws))
+        tdm_ws - True for WebSocket, else TCP (default: False)
+        password - TDM password for nonlocal connections (default: none)
         robot_id - robot node id (default: any)
         robot_name - robot name (default: any)
-        zeroconf - True to find TDM with zeroconf (default: automatic)
+        zeroconf - True to use zeroconf (default: false)
+        zeroconf_all - True to use zeroconf with all interfaces instead of default (default: false)
     """
 
-    client = ClientAsync(zeroconf=zeroconf,
-                         tdm_addr=tdm_addr, tdm_port=tdm_port,
+    client = ClientAsync(zeroconf=zeroconf, zeroconf_all=zeroconf_all,
+                         tdm_addr=tdm_addr, tdm_port=tdm_port, tdm_ws=tdm_ws,
                          password=password,
                          debug=debug)
     node = await client.wait_for_node(**kwargs)
@@ -271,6 +274,20 @@ import getopt
 import sys
 import shlex
 
+def var_option_to_var_statement(arg, is_aseba=False):
+    name, expr = arg.split("=", 1)
+    globals_dict = get_ipython().user_ns
+    value = eval(expr, globals_dict)
+    statement = "var " + name if is_aseba else name
+    if isinstance(value, builtins.list):
+        statement += "[] = " if is_aseba else " = "
+        statement += "[" + ", ".join([str(int(el)) for el in value]) + "]"
+    else:
+        statement += " = "
+        statement += str(int(value))
+    statement += "\n"
+    return statement
+
 @register_cell_magic
 def run_python(line, cell):
     """Transpile the whole cell from Python to Aseba, send it to the robot and
@@ -279,10 +296,10 @@ def run_python(line, cell):
     Options:
 
         --clear-event-data: clear event data so that function get_event_data()
-        doesn't include events from a previous run.
+                            doesn't include events from a previous run.
 
         --nothymio: don't import Thymio symbols (module "thymio" should be
-        imported explicitly if needed).
+                    imported explicitly if needed).
 
         --robotid ID: run on robot specified by id.
 
@@ -291,17 +308,25 @@ def run_python(line, cell):
         --robotindex I: run on robot specified by index
                         (0=first=default, 1=second etc.)
 
+        --var NAME=VALUE: define a variable where the initial value is
+                          evaluated in the context of the notebook.
+
         --wait: continue running to receive events from the robot and display
-        print output until exit() is called in the program or the execution
-        is interrupted. Other events are stored and can be obtained with
-        function get_event_data().
+                print output until exit() is called in the program or the
+                execution is interrupted. Other events are stored and can be
+                obtained with function get_event_data().
+
+        --warning-missing-global: display warnings for local variables which
+                                  hide global variables with the same name
     """
 
     args = shlex.split(line)
     wait = False
     clear_event_data = False
     import_thymio = True
+    warning_missing_global = False
     nodes = []
+
     try:
         arguments, values = getopt.getopt(args,
                                           "",
@@ -311,11 +336,14 @@ def run_python(line, cell):
                                               "robotid=",
                                               "robotindex=",
                                               "robotname=",
+                                              "var=",
                                               "wait",
+                                              "warning-missing-global",
                                           ])
     except getopt.error as err:
         print(str(err), file=sys.stderr)
         return
+    python_preamble = ""
     for arg, val in arguments:
         if arg == "--clear-event-data":
             clear_event_data = True
@@ -336,8 +364,12 @@ def run_python(line, cell):
                 node = _interactive_console.find_robot(robot_name=name)
                 if node not in nodes:
                     nodes.append(node)
+        elif arg == "--var":
+            python_preamble += var_option_to_var_statement(val)
         elif arg == "--wait":
             wait = True
+        elif arg == "--warning-missing-global":
+            warning_missing_global = True
     if len(values) > 0:
         print(f"Unexpected argument {values[0]}", file=sys.stderr)
         return
@@ -346,9 +378,10 @@ def run_python(line, cell):
         _interactive_console.clear_event_data()
 
     try:
-        _interactive_console.run_program(cell,
+        _interactive_console.run_program(python_preamble + cell,
                                          nodes=nodes if len(nodes) > 0 else None,
                                          language="python",
+                                         warning_missing_global=warning_missing_global,
                                          wait=wait, import_thymio=import_thymio)
     except KeyboardInterrupt:
         # avoid long exception message with stack trace
@@ -366,6 +399,10 @@ def run_aseba(line, cell):
 
         --robotindex I: run on robot specified by index
                         (0=first=default, 1=second etc.)
+
+        --var NAME=VALUE: define a variable where the initial value is
+                          evaluated in the context of the notebook.
+
     """
 
     args = shlex.split(line)
@@ -377,10 +414,12 @@ def run_aseba(line, cell):
                                               "robotid=",
                                               "robotindex=",
                                               "robotname=",
+                                              "var=",
                                           ])
     except getopt.error as err:
         print(str(err), file=sys.stderr)
         return
+    aseba_preamble = ""
     for arg, val in arguments:
         if arg == "--robotid":
             for id in val.split(","):
@@ -397,11 +436,13 @@ def run_aseba(line, cell):
                 node = _interactive_console.find_robot(robot_name=name)
                 if node not in nodes:
                     nodes.append(node)
+        elif arg == "--var":
+            aseba_preamble += var_option_to_var_statement(val, True)
     if len(values) > 0:
         print(f"Unexpected argument {values[0]}", file=sys.stderr)
         return
 
-    _interactive_console.run_program(cell,
+    _interactive_console.run_program(aseba_preamble + cell,
                                      nodes = nodes if len(nodes) > 0 else None,
                                      language="aseba")
 
@@ -412,27 +453,45 @@ def transpile_to_aseba(line, cell):
     Option:
 
         --nothymio: don't import Thymio symbols (module "thymio" should be
-        imported explicitly if needed).
+                    imported explicitly if needed).
+
+        --var NAME=VALUE: define a variable where the initial value is
+                          evaluated in the context of the notebook.
+
+        --warning-missing-global: display warnings for local variables which
+                                  hide global variables with the same name
     """
 
     args = shlex.split(line)
     import_thymio = True
+    warning_missing_global = False
+
     try:
         arguments, values = getopt.getopt(args,
                                           "",
                                           [
                                               "nothymio",
+                                              "var=",
+                                              "warning-missing-global",
                                           ])
     except getopt.error as err:
         print(str(err), file=sys.stderr)
         return
+    python_preamble = ""
     for arg, val in arguments:
         if arg == "--nothymio":
             import_thymio = False
+        elif arg == "--var":
+            python_preamble += var_option_to_var_statement(val)
+        elif arg == "--warning-missing-global":
+            warning_missing_global = True
     if len(values) > 0:
         print(f"Unexpected argument {values[0]}", file=sys.stderr)
         return
 
-    transpiler = _interactive_console.transpile(cell, import_thymio=import_thymio)
+    transpiler = _interactive_console.transpile(python_preamble + cell,
+                                                import_thymio=import_thymio,
+                                                warning_missing_global=warning_missing_global)
+
     src_a = transpiler.get_output()
     print(src_a)
