@@ -227,7 +227,7 @@ class Node(Listener):
             (
                 self.thymio.next_request_id(**kwargs),
                 (
-                    self.id,
+                    self.props["group_id"],
                 ),
                 [
                     (
@@ -328,6 +328,7 @@ class ThymioFB(Listener):
     WATCHABLE_INFO_SHARED_EVENTS_DESCRIPTION = 0x8
     WATCHABLE_INFO_SHARED_VARIABLES = 0x10
     WATCHABLE_INFO_SCRATCHPADS = 0x20
+    WATCHABLE_INFO_ALL = 0x3f
 
     PROGRAMMING_LANGUAGE_ASEBA = 1
     PROGRAMMING_LANGUAGE_AESL = 2
@@ -511,6 +512,16 @@ class ThymioFB(Listener):
         )
     """
 
+    ERROR_MSG_DICT = {
+        0: "no error",
+        1: "unknown error",
+        2: "unknown node",
+        3: "node busy",
+        4: "unsupported variable type",
+        5: "Thmyio II pairing write dongle failed",
+        6: "Thymio II pairing write robot failed",
+    }
+
     @staticmethod
     def create_message(msg, schema=None):
         fb = FlatBuffer()
