@@ -1,5 +1,5 @@
 # This file is part of tdmclient.
-# Copyright 2021 ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE,
+# Copyright 2021-2022 ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE,
 # Miniature Mobile Robots group, Switzerland
 # Author: Yves Piguet
 #
@@ -15,22 +15,27 @@ from tdmclient import ClientAsync
 from tdmclient.atranspiler import ATranspiler
 
 
-class VariableTableWindow(tk.Tk):
+class GUIWindow(tk.Tk):
 
     def __init__(self,
-                 zeroconf=None,
-                 tdm_addr=None, tdm_port=None, password=None,
+                 zeroconf=False, zeroconf_all=False,
+                 tdm_addr=None, tdm_port=None, tdm_ws=False,
+                 tdm_transport=None,
+                 password=None,
                  node_id=None, node_name=None,
                  language=None, debug=0):
-        super(VariableTableWindow, self).__init__()
+        super(GUIWindow, self).__init__()
         self.geometry("800x600")
 
         self.program_path = None
         self.program_src = ""
         self.language = language or "aseba"
         self.zeroconf = zeroconf
+        self.zeroconf_all = zeroconf_all
         self.tdm_addr = tdm_addr
         self.tdm_port = tdm_port
+        self.tdm_ws = tdm_ws
+        self.tdm_transport = tdm_transport
         self.password = password
         self.node_id = node_id
         self.node_name = node_name
@@ -192,10 +197,20 @@ class VariableTableWindow(tk.Tk):
 
         self.set_title()
 
+        # disconnect on exit
+        def on_exit():
+            self.disconnect()
+            self.destroy()
+
+        self.protocol("WM_DELETE_WINDOW", on_exit)
+
     def set_title(self):
         name = self.node.props["name"] if self.node is not None else "No robot"
         if self.client is not None and self.client.tdm_addr is not None:
-            name += f" (TDM: {self.client.tdm_addr}:{self.client.tdm_port})"
+            if self.tdm_transport:
+                name += " (TDM: alternative transport)"
+            else:
+                name += f" (TDM: {'ws://' if self.tdm_ws else ''}{self.client.tdm_addr}:{self.client.tdm_port})"
         if self.text_program is not None:
             name += " - "
             name += os.path.basename(self.program_path) if self.program_path is not None else f"Untitled.{self.language}"
@@ -480,8 +495,11 @@ class VariableTableWindow(tk.Tk):
                         self.add_variable(name, variables[name])
 
         self.client = ClientAsync(zeroconf=self.zeroconf,
+                                  zeroconf_all=self.zeroconf_all,
                                   tdm_addr=self.tdm_addr,
                                   tdm_port=self.tdm_port,
+                                  tdm_ws=self.tdm_ws,
+                                  tdm_transport=self.tdm_transport,
                                   password=self.password,
                                   debug=self.debug)
         self.client.on_nodes_changed = on_nodes_changed
@@ -489,90 +507,104 @@ class VariableTableWindow(tk.Tk):
         # schedule communication
         self.after(100, self.run)
 
+    def disconnect(self):
+        if self.client is not None:
+            self.client.disconnect()
+            self.client = None
+
     def run(self):
         if self.start_co is not None:
             if not self.client.step_coroutine(self.start_co):
                 # start_co is finished
                 self.start_co = None
-        else:
+        elif self.client is not None:
             self.client.process_waiting_messages()
         self.after(100, self.run)
 
 
 def help():
-    print("""Usage: python3 -m tdmclient.tools.variables [options]
+    print(f"""Usage: python3 -m tdmclient gui [options]
 Variable browser and code editor
 
 Options:
   --debug n      display diagnostic information (0=none, 1=basic, 2=more, 3=verbose)
   --help         display this help message and exit
   --language=L   programming language (aseba or python); default=automatic
-  --nozeroconf   don't use zeroconf (default: automatic)
   --password=PWD specify password for remote tdm
   --robotid=I    robot id; default=any
   --robotname=N  robot name; default=any
   --tdmaddr=H    tdm address (default: localhost or from zeroconf)
-  --tdmport=P    tdm port (default: from zeroconf)
-  --zeroconf     use zeroconf (default: automatic)
+  --tdmport=P    tdm port (default: 8596 (tcp) or 8597 (ws), or from zeroconf)
+  --tdmws        connect to tdm with WebSocket (default: plain TCP)
+  --zeroconf     use zeroconf (default: no zeroconf)
+  --zcall        discover TDM information published on all interfaces instead
+                 of only default one
 """)
 
 
-if __name__ == "__main__":
-
+def main(argv=None, tdm_transport=None):
     debug = 0
     language = None  # auto
-    zeroconf = None  # auto
+    zeroconf = False
+    zeroconf_all = False
     tdm_addr = None
     tdm_port = None
+    tdm_ws = False
     password = None
     robot_id = None
     robot_name = None
 
-    try:
-        arguments, values = getopt.getopt(sys.argv[1:],
-                                          "",
-                                          [
-                                              "debug=",
-                                              "help",
-                                              "language=",
-                                              "nozeroconf",
-                                              "password=",
-                                              "robotid=",
-                                              "robotname=",
-                                              "tdmaddr=",
-                                              "tdmport=",
-                                              "zeroconf",
-                                          ])
-    except getopt.error as err:
-        print(str(err))
-        sys.exit(1)
-    for arg, val in arguments:
-        if arg == "--help":
-            help()
-            sys.exit(0)
-        elif arg == "--debug":
-            debug = int(val)
-        elif arg == "--language":
-            language = val
-        elif arg == "--nozeroconf":
-            zeroconf = False
-        elif arg == "--password":
-            password = val
-        elif arg == "--robotid":
-            robot_id = val
-        elif arg == "--robotname":
-            robot_name = val
-        elif arg == "--tdmaddr":
-            tdm_addr = val
-        elif arg == "--tdmport":
-            tdm_port = int(val)
-        elif arg == "--zeroconf":
-            zeroconf = True
+    if argv is not None:
+        try:
+            arguments, values = getopt.getopt(argv[1:],
+                                              "",
+                                              [
+                                                  "debug=",
+                                                  "help",
+                                                  "language=",
+                                                  "password=",
+                                                  "robotid=",
+                                                  "robotname=",
+                                                  "tdmaddr=",
+                                                  "tdmport=",
+                                                  "tdmws",
+                                                  "zcall",
+                                                  "zeroconf",
+                                              ])
+        except getopt.error as err:
+            print(str(err))
+            return 1
+        for arg, val in arguments:
+            if arg == "--help":
+                help()
+                return 0
+            elif arg == "--debug":
+                debug = int(val)
+            elif arg == "--language":
+                language = val
+            elif arg == "--password":
+                password = val
+            elif arg == "--robotid":
+                robot_id = val
+            elif arg == "--robotname":
+                robot_name = val
+            elif arg == "--tdmaddr":
+                tdm_addr = val
+            elif arg == "--tdmport":
+                tdm_port = ClientAsync.DEFAULT_TDM_PORT if val == "default" else int(val)
+            elif arg == "--tdmws":
+                tdm_ws = True
+            elif arg == "--zeroconf":
+                zeroconf = True
+            elif arg == "--zcall":
+                zeroconf = True
+                zeroconf_all = True
 
-    win = VariableTableWindow(zeroconf=zeroconf,
-                              tdm_addr=tdm_addr, tdm_port=tdm_port,
-                              password=password,
-                              node_id=robot_id, node_name=robot_name,
-                              language=language, debug=debug)
+    win = GUIWindow(zeroconf=zeroconf, zeroconf_all=zeroconf_all,
+                    tdm_addr=tdm_addr, tdm_port=tdm_port, tdm_ws=tdm_ws,
+                    tdm_transport=tdm_transport,
+                    password=password,
+                    node_id=robot_id, node_name=robot_name,
+                    language=language, debug=debug)
     win.connect()
     win.mainloop()
