@@ -1,5 +1,5 @@
 # This file is part of tdmclient.
-# Copyright 2021 ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE,
+# Copyright 2021-2022 ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE,
 # Miniature Mobile Robots group, Switzerland
 # Author: Yves Piguet
 #
@@ -21,7 +21,7 @@ except ModuleNotFoundError:
 
 
 def help():
-    print("""Usage: python3 -m tdmclient.tools.repl [options]
+    print(f"""Usage: python3 -m tdmclient repl [options]
 Read-eval-print loop with link and synchronization with Thymio
 
 Options:
@@ -30,13 +30,20 @@ Options:
   --robotid=I    robot id; default=any
   --robotname=N  robot name; default=any
   --tdmaddr=H    tdm address (default: localhost or from zeroconf)
-  --tdmport=P    tdm port (default: from zeroconf)
+  --tdmport=P    tdm port (default: 8596 (tcp) or 8597 (ws), or from zeroconf)
+  --tdmws        connect to tdm with WebSocket (default: plain TCP)
+  --zeroconf     use zeroconf (default: no zeroconf)
+  --zcall        discover TDM information published on all interfaces instead
+                 of only default one
 """)
 
 
-def main(argv=None):
+def main(argv=None, tdm_transport=None):
+    zeroconf = False
+    zeroconf_all = False
     tdm_addr = None
     tdm_port = None
+    tdm_ws = False
     password = None
     robot_id = None
     robot_name = None
@@ -52,14 +59,17 @@ def main(argv=None):
                                                   "robotname=",
                                                   "tdmaddr=",
                                                   "tdmport=",
+                                                  "tdmws",
+                                                  "zcall",
+                                                  "zeroconf",
                                               ])
         except getopt.error as err:
             print(str(err))
-            sys.exit(1)
+            return 1
         for arg, val in arguments:
             if arg == "--help":
                 help()
-                sys.exit(0)
+                return 0
             elif arg == "--password":
                 password = val
             elif arg == "--robotid":
@@ -69,9 +79,18 @@ def main(argv=None):
             elif arg == "--tdmaddr":
                 tdm_addr = val
             elif arg == "--tdmport":
-                tdm_port = int(val)
+                tdm_port = ClientAsync.DEFAULT_TDM_PORT if val == "default" else int(val)
+            elif arg == "--tdmws":
+                tdm_ws = True
+            elif arg == "--zeroconf":
+                zeroconf = True
+            elif arg == "--zcall":
+                zeroconf = True
+                zeroconf_all = True
 
-    with ClientAsync(tdm_addr=tdm_addr, tdm_port=tdm_port,
+    with ClientAsync(zeroconf=zeroconf, zeroconf_all=zeroconf_all,
+                     tdm_addr=tdm_addr, tdm_port=tdm_port, tdm_ws=tdm_ws,
+                     tdm_transport=tdm_transport,
                      password=password) as client:
 
         async def co_init():
@@ -84,7 +103,3 @@ def main(argv=None):
                 interactive_console.interact()
 
         client.run_async_program(co_init)
-
-
-if __name__ == "__main__":
-    main(sys.argv)
